@@ -9,10 +9,10 @@ pub use gemachain_program::log::*;
 )]
 macro_rules! info {
     ($msg:expr) => {
-        $crate::log::sol_log($msg)
+        $crate::log::gema_log($msg)
     };
     ($arg1:expr, $arg2:expr, $arg3:expr, $arg4:expr, $arg5:expr) => {
-        $crate::log::sol_log_64(
+        $crate::log::gema_log_64(
             $arg1 as u64,
             $arg2 as u64,
             $arg3 as u64,
