@@ -1,8 +1,14 @@
+#![cfg(feature = "full")]
+
 use itertools::Itertools;
 use gemachain_sdk::{
     account::AccountSharedData,
+    compute_budget::ComputeBudget,
+    fee_calculator::FeeCalculator,
+    hash::Hash,
     instruction::{CompiledInstruction, Instruction, InstructionError},
     keyed_account::{create_keyed_accounts_unified, KeyedAccount},
+    message::Message,
     pubkey::Pubkey,
     sysvar::Sysvar,
 };
@@ -46,16 +52,15 @@ impl<'a> InvokeContextStackFrame<'a> {
 /// Invocation context passed to loaders
 pub trait InvokeContext {
     /// Push a stack frame onto the invocation stack
-    ///
-    /// Used in MessageProcessor::process_cross_program_instruction
     fn push(
         &mut self,
         key: &Pubkey,
-        keyed_accounts: &[(bool, bool, &Pubkey, &RefCell<AccountSharedData>)],
+        message: &Message,
+        instruction: &CompiledInstruction,
+        program_indices: &[usize],
+        account_indices: &[usize],
     ) -> Result<(), InstructionError>;
     /// Pop a stack frame from the invocation stack
-    ///
-    /// Used in MessageProcessor::process_cross_program_instruction
     fn pop(&mut self);
     /// Current depth of the invocation stake
     fn invoke_depth(&self) -> usize;
@@ -63,7 +68,7 @@ pub trait InvokeContext {
     fn verify_and_update(
         &mut self,
         instruction: &CompiledInstruction,
-        accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
+        account_indices: &[usize],
         write_privileges: &[bool],
     ) -> Result<(), InstructionError>;
     /// Get the program ID of the currently executing program
@@ -77,6 +82,8 @@ pub trait InvokeContext {
     /// Get this invocation's logger
     fn get_logger(&self) -> Rc<RefCell<dyn Logger>>;
     /// Get this invocation's compute budget
+    #[allow(deprecated)]
+    #[deprecated(since = "1.8.0", note = "please use `get_compute_budget` instead")]
     fn get_bpf_compute_budget(&self) -> &BpfComputeBudget;
     /// Get this invocation's compute meter
     fn get_compute_meter(&self) -> Rc<RefCell<dyn ComputeMeter>>;
@@ -89,8 +96,8 @@ pub trait InvokeContext {
     fn record_instruction(&self, instruction: &Instruction);
     /// Get the bank's active feature set
     fn is_feature_active(&self, feature_id: &Pubkey) -> bool;
-    /// Get an account by its key
-    fn get_account(&self, pubkey: &Pubkey) -> Option<Rc<RefCell<AccountSharedData>>>;
+    /// Find an account_index and account by its key
+    fn get_account(&self, pubkey: &Pubkey) -> Option<(usize, Rc<RefCell<AccountSharedData>>)>;
     /// Update timing
     fn update_timing(
         &mut self,
@@ -101,6 +108,12 @@ pub trait InvokeContext {
     );
     /// Get sysvar data
     fn get_sysvar_data(&self, id: &Pubkey) -> Option<Rc<Vec<u8>>>;
+    /// Get this invocation's compute budget
+    fn get_compute_budget(&self) -> &ComputeBudget;
+    /// Get this invocation's blockhash
+    fn get_blockhash(&self) -> &Hash;
+    /// Get this invocation's `FeeCalculator`
+    fn get_fee_calculator(&self) -> &FeeCalculator;
     /// Set the return data
     fn set_return_data(&mut self, return_data: Option<(Pubkey, Vec<u8>)>);
     /// Get the return data
@@ -152,13 +165,12 @@ pub fn get_sysvar<T: Sysvar>(
     })
 }
 
-#[derive(Clone, Copy, Debug, AbiExample, PartialEq)]
+#[deprecated(since = "1.8.0", note = "please use `ComputeBudget` instead")]
+#[derive(Clone, Copy, Debug, PartialEq)]
 pub struct BpfComputeBudget {
     /// Number of compute units that an instruction is allowed.  Compute units
     /// are consumed by program execution, resources they use, etc...
     pub max_units: u64,
-    /// Number of compute units consumed by a log call
-    pub log_units: u64,
     /// Number of compute units consumed by a log_u64 call
     pub log_64_units: u64,
     /// Number of compute units consumed by a create_program_address call
@@ -190,38 +202,63 @@ pub struct BpfComputeBudget {
     pub syscall_base_cost: u64,
     /// Optional program heap region size, if `None` then loader default
     pub heap_size: Option<usize>,
-    /// Number of compute units per additional 32k heap above the default (~.5
-    /// us per 32k at 15 units/us rounded up)
-    pub heap_cost: u64,
 }
-
-impl Default for BpfComputeBudget {
-    fn default() -> Self {
-        Self::new()
+#[allow(deprecated)]
+impl From<ComputeBudget> for BpfComputeBudget {
+    fn from(item: ComputeBudget) -> Self {
+        BpfComputeBudget {
+            max_units: item.max_units,
+            log_64_units: item.log_64_units,
+            create_program_address_units: item.create_program_address_units,
+            invoke_units: item.invoke_units,
+            max_invoke_depth: item.max_invoke_depth,
+            sha256_base_cost: item.sha256_base_cost,
+            sha256_byte_cost: item.sha256_byte_cost,
+            max_call_depth: item.max_call_depth,
+            stack_frame_size: item.stack_frame_size,
+            log_pubkey_units: item.log_pubkey_units,
+            max_cpi_instruction_size: item.max_cpi_instruction_size,
+            cpi_bytes_per_unit: item.cpi_bytes_per_unit,
+            sysvar_base_cost: item.sysvar_base_cost,
+            syscall_base_cost: item.syscall_base_cost,
+            secp256k1_recover_cost: item.secp256k1_recover_cost,
+            heap_size: item.heap_size,
+        }
     }
 }
+#[allow(deprecated)]
+impl From<BpfComputeBudget> for ComputeBudget {
+    fn from(item: BpfComputeBudget) -> Self {
+        ComputeBudget {
+            max_units: item.max_units,
+            log_64_units: item.log_64_units,
+            create_program_address_units: item.create_program_address_units,
+            invoke_units: item.invoke_units,
+            max_invoke_depth: item.max_invoke_depth,
+            sha256_base_cost: item.sha256_base_cost,
+            sha256_byte_cost: item.sha256_byte_cost,
+            max_call_depth: item.max_call_depth,
+            stack_frame_size: item.stack_frame_size,
+            log_pubkey_units: item.log_pubkey_units,
+            max_cpi_instruction_size: item.max_cpi_instruction_size,
+            cpi_bytes_per_unit: item.cpi_bytes_per_unit,
+            sysvar_base_cost: item.sysvar_base_cost,
+            secp256k1_recover_cost: item.secp256k1_recover_cost,
+            syscall_base_cost: item.syscall_base_cost,
+            heap_size: item.heap_size,
+        }
+    }
+}
+#[allow(deprecated)]
+impl Default for BpfComputeBudget {
+    fn default() -> Self {
+        ComputeBudget::default().into()
+    }
+}
+#[allow(deprecated)]
 impl BpfComputeBudget {
     pub fn new() -> Self {
-        BpfComputeBudget {
-            max_units: 200_000,
-            log_units: 100,
-            log_64_units: 100,
-            create_program_address_units: 1500,
-            invoke_units: 1000,
-            max_invoke_depth: 4,
-            sha256_base_cost: 85,
-            sha256_byte_cost: 1,
-            max_call_depth: 64,
-            stack_frame_size: 4_096,
-            log_pubkey_units: 100,
-            max_cpi_instruction_size: 1280, // IPv6 Min MTU size
-            cpi_bytes_per_unit: 250,        // ~50MB at 200,000 units
-            sysvar_base_cost: 100,
-            syscall_base_cost: 100,
-            secp256k1_recover_cost: 25_000,
-            heap_size: None,
-            heap_cost: 8,
-        }
+        BpfComputeBudget::default()
     }
 }
 
@@ -256,7 +293,10 @@ pub mod stable_log {
     /// Log a program invoke.
     ///
     /// The general form is:
-    ///     "Program <address> invoke [<depth>]"
+    ///
+    /// ```notrust
+    /// "Program <address> invoke [<depth>]"
+    /// ```
     pub fn program_invoke(
         logger: &Rc<RefCell<dyn Logger>>,
         program_id: &Pubkey,
@@ -268,7 +308,11 @@ pub mod stable_log {
     /// Log a message from the program itself.
     ///
     /// The general form is:
-    ///     "Program log: <program-generated output>"
+    ///
+    /// ```notrust
+    /// "Program log: <program-generated output>"
+    /// ```
+    ///
     /// That is, any program-generated output is guaranteed to be prefixed by "Program log: "
     pub fn program_log(logger: &Rc<RefCell<dyn Logger>>, message: &str) {
         ic_logger_msg!(logger, "Program log: {}", message);
@@ -313,7 +357,10 @@ pub mod stable_log {
     /// Log successful program execution.
     ///
     /// The general form is:
-    ///     "Program <address> success"
+    ///
+    /// ```notrust
+    /// "Program <address> success"
+    /// ```
     pub fn program_success(logger: &Rc<RefCell<dyn Logger>>, program_id: &Pubkey) {
         ic_logger_msg!(logger, "Program {} success", program_id);
     }
@@ -321,7 +368,10 @@ pub mod stable_log {
     /// Log program execution failure
     ///
     /// The general form is:
-    ///     "Program <address> failed: <program error details>"
+    ///
+    /// ```notrust
+    /// "Program <address> failed: <program error details>"
+    /// ```
     pub fn program_failure(
         logger: &Rc<RefCell<dyn Logger>>,
         program_id: &Pubkey,
@@ -375,25 +425,30 @@ impl Logger for MockLogger {
     }
 }
 
+#[allow(deprecated)]
 pub struct MockInvokeContext<'a> {
     pub invoke_stack: Vec<InvokeContextStackFrame<'a>>,
     pub logger: MockLogger,
+    pub compute_budget: ComputeBudget,
     pub bpf_compute_budget: BpfComputeBudget,
     pub compute_meter: MockComputeMeter,
     pub programs: Vec<(Pubkey, ProcessInstructionWithContext)>,
     pub accounts: Vec<(Pubkey, Rc<RefCell<AccountSharedData>>)>,
     pub sysvars: Vec<(Pubkey, Option<Rc<Vec<u8>>>)>,
     pub disabled_features: HashSet<Pubkey>,
+    pub blockhash: Hash,
+    pub fee_calculator: FeeCalculator,
     pub return_data: Option<(Pubkey, Vec<u8>)>,
 }
 
 impl<'a> MockInvokeContext<'a> {
     pub fn new(keyed_accounts: Vec<KeyedAccount<'a>>) -> Self {
-        let bpf_compute_budget = BpfComputeBudget::default();
+        let compute_budget = ComputeBudget::default();
         let mut invoke_context = MockInvokeContext {
-            invoke_stack: Vec::with_capacity(bpf_compute_budget.max_invoke_depth),
+            invoke_stack: Vec::with_capacity(compute_budget.max_invoke_depth),
             logger: MockLogger::default(),
-            bpf_compute_budget,
+            compute_budget,
+            bpf_compute_budget: compute_budget.into(),
             compute_meter: MockComputeMeter {
                 remaining: std::i64::MAX as u64,
             },
@@ -401,6 +456,8 @@ impl<'a> MockInvokeContext<'a> {
             accounts: vec![],
             sysvars: vec![],
             disabled_features: HashSet::default(),
+            blockhash: Hash::default(),
+            fee_calculator: FeeCalculator::default(),
             return_data: None,
         };
         invoke_context
@@ -431,15 +488,15 @@ pub fn mock_set_sysvar<T: Sysvar>(
 impl<'a> InvokeContext for MockInvokeContext<'a> {
     fn push(
         &mut self,
-        key: &Pubkey,
-        keyed_accounts: &[(bool, bool, &Pubkey, &RefCell<AccountSharedData>)],
+        _key: &Pubkey,
+        _message: &Message,
+        _instruction: &CompiledInstruction,
+        _program_indices: &[usize],
+        _account_indices: &[usize],
     ) -> Result<(), InstructionError> {
-        fn transmute_lifetime<'a, 'b>(value: Vec<KeyedAccount<'a>>) -> Vec<KeyedAccount<'b>> {
-            unsafe { std::mem::transmute(value) }
-        }
         self.invoke_stack.push(InvokeContextStackFrame::new(
-            *key,
-            transmute_lifetime(create_keyed_accounts_unified(keyed_accounts)),
+            *_key,
+            create_keyed_accounts_unified(&[]),
         ));
         Ok(())
     }
@@ -452,7 +509,7 @@ impl<'a> InvokeContext for MockInvokeContext<'a> {
     fn verify_and_update(
         &mut self,
         _instruction: &CompiledInstruction,
-        _accounts: &[(Pubkey, Rc<RefCell<AccountSharedData>>)],
+        _account_indices: &[usize],
         _write_pivileges: &[bool],
     ) -> Result<(), InstructionError> {
         Ok(())
@@ -484,7 +541,9 @@ impl<'a> InvokeContext for MockInvokeContext<'a> {
     fn get_logger(&self) -> Rc<RefCell<dyn Logger>> {
         Rc::new(RefCell::new(self.logger.clone()))
     }
+    #[allow(deprecated)]
     fn get_bpf_compute_budget(&self) -> &BpfComputeBudget {
+        #[allow(deprecated)]
         &self.bpf_compute_budget
     }
     fn get_compute_meter(&self) -> Rc<RefCell<dyn ComputeMeter>> {
@@ -498,10 +557,10 @@ impl<'a> InvokeContext for MockInvokeContext<'a> {
     fn is_feature_active(&self, feature_id: &Pubkey) -> bool {
         !self.disabled_features.contains(feature_id)
     }
-    fn get_account(&self, pubkey: &Pubkey) -> Option<Rc<RefCell<AccountSharedData>>> {
-        for (key, account) in self.accounts.iter() {
+    fn get_account(&self, pubkey: &Pubkey) -> Option<(usize, Rc<RefCell<AccountSharedData>>)> {
+        for (index, (key, account)) in self.accounts.iter().enumerate().rev() {
             if key == pubkey {
-                return Some(account.clone());
+                return Some((index, account.clone()));
             }
         }
         None
@@ -519,6 +578,15 @@ impl<'a> InvokeContext for MockInvokeContext<'a> {
             .iter()
             .find_map(|(key, sysvar)| if id == key { sysvar.clone() } else { None })
     }
+    fn get_compute_budget(&self) -> &ComputeBudget {
+        &self.compute_budget
+    }
+    fn get_blockhash(&self) -> &Hash {
+        &self.blockhash
+    }
+    fn get_fee_calculator(&self) -> &FeeCalculator {
+        &self.fee_calculator
+    }
     fn set_return_data(&mut self, return_data: Option<(Pubkey, Vec<u8>)>) {
         self.return_data = return_data;
     }
